package com.smppw.common.pojo.enums.strategy;

import java.util.Arrays;
import java.util.List;
import java.util.stream.Stream;

// 新策略 - 第二级策略
public enum NewSubStrategy implements IStrategy {
    OtherStrategy(-1, "其他策略", "", "IN00000008"), /**/
    EquityLongOnly(100101, "主观多头", "IN000002FP", "IN00000008"), /**/
    EquityQuantitative(100102, "量化多头", "IN000002FQ", "IN00000008"), /**/
    EquityLongShort(100103, "股票多空", "IN000002FR", "IN00000008"), /**/
    EquityNeutral(100104, "股票市场中性", "IN000002FS", "IN00000008"), /**/
    BondPure(100201, "纯债策略", "IN000002FT", "IN00000077"), /**/
    BondStrong(100202, "债券增强", "IN000002FU", "IN00000077"), /**/
    BondMultiple(100203, "债券复合", "IN000002FV", "IN00000077"), /**/
    CnvtbBondTrd(100204, "转债交易策略", "IN000002FW", "IN0000008J"),
    ActMgtFut(100301, "主观CTA", "IN000002FX", "IN0000008S"),
    MgtFutQuant(100302, "量化CTA", "IN000002FY", "IN0000008S"),
    OptionStrgy(100303, "期权策略", "IN000002FZ", "IN0000008S"),
    //    OtherFutures(100304, "其他衍生品策略", "IN000002DI", "IN0000008S"),
    MacroStrgy(100401, "宏观策略", "IN000002G0", "IN00000008"),
    ArbitrageStrgy(100402, "套利策略", "IN000002G1", "IN00000008"),
    CombinedStrgy(100403, "复合策略", "IN000002G2", "IN00000008"),
    SubFOF(100501, "FOF", "IN000002G3", "IN00000008"), /**/
    MOM(100502, "MOM", "IN000002G4", "IN00000008"),
    ; /**/


    public static final List<NewSubStrategy> STRATEGY_LIST = Arrays.asList(EquityLongOnly, EquityQuantitative, EquityLongShort, EquityNeutral,
            BondPure, BondStrong,
            BondMultiple, CnvtbBondTrd, ActMgtFut, MgtFutQuant, OptionStrgy, MacroStrgy, ArbitrageStrgy, CombinedStrgy, SubFOF, MOM);

    private Integer id;
    private String nameDesc;
    private String rongZhiIndex;
    private String benchmark;

    private NewSubStrategy(Integer id, String nameDesc, String rongZhiIndex, String benchmark) {
        this.id = id;
        this.nameDesc = nameDesc;
        this.rongZhiIndex = rongZhiIndex;
        this.benchmark = benchmark;
    }

    public static NewSubStrategy getSubStrategy(int id) {
        return Stream.of(NewSubStrategy.values()).filter(e -> e.id == id).findFirst().orElse(null);
    }

    public static NewSubStrategy getStrategyByName(String name) {
        if (null == name) {
            return null;
        }
        for (NewSubStrategy s : NewSubStrategy.values()) {
            if (name.equals(s.getNameDesc() + "")) {
                return s;
            }
        }
        return null;
    }

    public static NewSubStrategy getStrategyByN(String name) {
        if (null == name) {
            return null;
        }
        for (NewSubStrategy s : NewSubStrategy.values()) {
            if (name.equals(s.name())) {
                return s;
            }
        }
        return null;
    }

    public static NewSubStrategy getStrategy(Integer id) {
        return Stream.of(NewSubStrategy.values()).filter(e -> e.id.equals(id)).findFirst().orElse(null);
    }

    public Integer getId() {
        return id;
    }

    public String getNameDesc() {
        return nameDesc;
    }

    public String getRongZhiIndex() {
        return rongZhiIndex;
    }

    public String getBenchmark() {
        return benchmark;
    }

    @Override
    public Integer getStrategyId() {
        return this.getId();
    }

    @Override
    public String getStrategyOriginName() {
        return this.name();
    }

    @Override
    public String getStrategyNameDesc() {
        return this.getNameDesc();
    }
}
