package com.smppw.core.rate.calc;


import cn.hutool.core.collection.ListUtil;
import com.smppw.constants.IndicatorConst;
import com.smppw.common.pojo.dto.CompoundRet;
import com.smppw.common.pojo.enums.Frequency;
import com.smppw.core.rate.Rate;
import com.smppw.utils.DataUtil;
import com.smppw.utils.MathUtils;

import java.util.ArrayList;
import java.util.List;

public class Kappa extends Rate {

	@Override
	public Double calc(List<CompoundRet> list, Frequency frequency, boolean ifAnnualized) {
		double LPM3 = 0.0;
		List<Double> rates = new ArrayList<>();
		List<Double> rfs = new ArrayList<>();
		List<String> dateList = ListUtil.list(true);
		for (CompoundRet value : list) {
			if (value.getRetOfFund() == null || value.getRetOfRf() == null) {
				dateList.add(value.getEndDate());
				continue;
			}
			double max = value.getRetOfRf() - value.getRetOfFund();
			if (max < 0) {
				max = 0;
			}
			LPM3 += Math.pow(max, 3);
			rates.add(value.getRetOfFund());
			rfs.add(value.getRetOfRf());
			dateList.add(value.getEndDate());
		}
		Double ratesAnnual = MathUtils.getAnnualReturn(rates, dateList, true);
		Double rfsAnnual = MathUtils.getAnnualReturn(rfs, dateList, true);

		if (list.size() <= 1 || ratesAnnual == null || rfsAnnual == null) {
			return null;
		}

		double minusValue = ratesAnnual - rfsAnnual;
		LPM3 /= list.size() - 1;
		LPM3 = Math.pow(LPM3 * frequency.getNum(), 1.0 / 3);
		if (Math.abs(LPM3) <= IndicatorConst.MIN_DENOMINATOR_VALUE) {
			// 如果分子也是0，结果无意义
			if (minusValue == 0.) {
				return null;
			} else if (minusValue < 0.) {
				return IndicatorConst.BEST_INDICATOR_VALUE * -1;
			}
			return IndicatorConst.BEST_INDICATOR_VALUE;
		}
		return DataUtil.divNearZeroValue(minusValue, LPM3);
	}

}
