package com.smppw.core.rate.calc;


import com.smppw.common.pojo.dto.CompoundRet;
import com.smppw.common.pojo.enums.Frequency;
import com.smppw.constants.IndicatorConst;
import com.smppw.core.rate.Rate;
import com.smppw.utils.DataUtil;

import java.util.List;

public class SortinoRatio extends Rate {

	@Override
	public Double calc(List<CompoundRet> list, Frequency frequency, boolean ifAnnualized) {
		Double SortinoRatio = 0.0;
		int count = 0;
		for (CompoundRet value : list) {
			if (value.getRetOfFund() == null || value.getRetOfRf() == null) {
				continue;
			}
			count++;
			SortinoRatio += value.getRetOfFund() - value.getRetOfRf();
		}
		if (count == 0) {
			return null;
		}
		SortinoRatio /= count;

		Double downsideStdDev = new DownsideStdDev().calc(list, frequency, false);

		if (downsideStdDev != null && (downsideStdDev.compareTo(0d) == 0 || Double.isNaN(downsideStdDev))) {
			// 如果分子也是0，结果无意义
			if (SortinoRatio == 0.) {
				return null;
			} else if (SortinoRatio < 0.) {
				return IndicatorConst.BEST_INDICATOR_VALUE * -1;
			}
			return IndicatorConst.BEST_INDICATOR_VALUE;
		}

		SortinoRatio = DataUtil.divNearZeroValue(SortinoRatio, downsideStdDev);
		if (SortinoRatio == null) {
			return null;
		}
		if (ifAnnualized) {
			SortinoRatio = Rate.AnnualizedBySqrt(SortinoRatio, frequency);
		}

		return DataUtil.handleOverflowData(SortinoRatio);
	}
}
