package com.smppw.core.rate.calc;


import cn.hutool.core.collection.ListUtil;
import com.smppw.common.pojo.dto.CompoundRet;
import com.smppw.common.pojo.enums.Frequency;
import com.smppw.constants.IndicatorConst;
import com.smppw.core.rate.Rate;
import com.smppw.utils.DataUtil;
import com.smppw.utils.MathUtils;

import java.util.ArrayList;
import java.util.List;

public class TreynorRatio extends Rate {

	@Override
	public Double calc(List<CompoundRet> list, Frequency frequency, boolean ifAnnualized) {
		List<String> dateList = ListUtil.list(true);
		List<Double> rates = new ArrayList<>();
		List<Double> rfs = new ArrayList<>();
		for (CompoundRet value : list) {
			if (value.getRetOfFund() == null && value.getRetOfRf() != null) {
				dateList.add(value.getEndDate());
				continue;
			}
			rates.add(value.getRetOfFund());
			rfs.add(value.getRetOfRf());
			dateList.add(value.getEndDate());
		}
		Double ratesAnnual = MathUtils.getAnnualReturn(rates, dateList, true);
		Double rfsAnnual = MathUtils.getAnnualReturn(rfs, dateList, true);
		Double beta = new Beta().calc(list, frequency, ifAnnualized);

		if (beta == null || rfsAnnual == null || ratesAnnual == null) {
			return null;
		}
		double minusValue = ratesAnnual - rfsAnnual;
		if (beta.compareTo(0d) == 0 || Double.isNaN(beta)) {
			// 如果分子也是0，结果无意义
			if (minusValue == 0.) {
				return null;
			} else if (minusValue < 0.) {
				return IndicatorConst.BEST_INDICATOR_VALUE * -1;
			}
			return IndicatorConst.BEST_INDICATOR_VALUE;
		}
		return DataUtil.divNearZeroValue(minusValue, beta);
	}

}
